/*
  S4.cpp - Library for the overall S4 Program
  Kevin Zack
  Sonoma State University
  NASA E/PO
  Released into the public domain.
*/
  #include "S4.h"
  #include "Arduino.h"
  #include "WiFly.h" 
  #include "NetworkParameters.h"
  #include "../S4GPS/S4GPS.h"
  #include "../S4GPS/S4GPS.cpp" 
  #include "Wire.h"  
  #include "SoftwareSerial.h"

  #define microRX 5 //datalines for the sd card writer
  #define microTX 4
  #define buadRate 9600
  
  #define BUFFSIZE 90 // buffer for GPS 
  
  S4GPS S4GPS;  

  bool WiFiIsOn = true; // used to turn on or off the WiFi connection
  bool gpsIsOn = true; // used to turn on or off the GPS 
  char gps[BUFFSIZE];
  String sensorData;
  
  
  SoftwareSerial microSerial = SoftwareSerial(microRX, microTX); 
   NetworkParameters parameters;

  S4::S4(){	 } 
  void S4::inheritValues()
  {	    
	 
  }


  void S4::useGPS(bool setUseGPS)
  {
       gpsIsOn = setUseGPS;
  }   
  void S4::useWiFi(bool statement)
  {
      WiFiIsOn = statement;
  }
  
  void S4::begin()
  {	 
	   inheritValues();	
	   Serial.println("Starting Main Program");	 	   
	   WDTCSR |= (1 << WDCE) | (1 << WDE);
       WDTCSR = 0;
       microSerial.begin(buadRate);  // this had to be before the gps
       Wire.begin(); 
       
       S4GPS.begin(buadRate);
       
       if(WiFiIsOn)
       {
		  WiFly.begin();		  
		  Serial.print("Attempting Association on Network ");
		  Serial.print(parameters.getValue(parameters.SSID).c_str());
		  Serial.println("...");
		  if(!WiFly.join(parameters.getValue(parameters.SSID).c_str()))
		  {
			  Serial.println("Network Association Failed");			 
		  }
		  else
		  {
			  Serial.println("Network Association Succeeded");
		  }
       }
      
       sensorData = "<sensor>";	  
       sensorData += parameters.getValue(parameters.DeviceName).c_str();     
      
  }  
  bool wait;    
  void S4::writeData()
  {
      //Serial.print("Remaining Memory:  ");
	  //Serial.println(memoryFree());
	  wait = true;
      while(wait)
      {
          if(gpsIsOn)
          {
              WriteGPS();
          }
          else
          {
              WriteSensors();
          }
      }
  } 
  void S4::WriteGPS()
  {
	if(S4GPS.getGPS(gps))
	{   
		sensorData += "</sensor>";                 
                 
		Serial.print("<gps>");
		Serial.print( parameters.getValue(parameters.DeviceName).c_str());
		Serial.print(",");  
		Serial.print(gps);
		Serial.println("</gps>");
		Serial.println(sensorData);                  
                  
		microSerial.print("<gps>");
		microSerial.print( parameters.getValue(parameters.DeviceName).c_str());
		microSerial.print(",");
		microSerial.print(gps);
		microSerial.println("</gps>");
		microSerial.println(sensorData);
                  
		if(WiFiIsOn)
		{
			SpiSerial.print("<gps>");
			SpiSerial.print( parameters.getValue(parameters.DeviceName).c_str());
			SpiSerial.print(",");  
			SpiSerial.print(gps);
			SpiSerial.println("</gps>");
			SpiSerial.println(sensorData);
		}
		sensorData = "<sensor>";
		sensorData +=  parameters.getValue(parameters.DeviceName).c_str();
		wait = false;
	}
  }
  void S4::WriteSensors()
  {
	    sensorData += "</sensor>";
		Serial.println(sensorData);
		microSerial.println(sensorData);
		if(WiFiIsOn)
		{
			SpiSerial.println(sensorData);
		} 
		sensorData = "<sensor>";
		sensorData += parameters.getValue(parameters.DeviceName).c_str();
		delay(1000);
		wait = false; 
  }
 
  void S4::addData(char* key, int value)
  {      
	  sensorData += ",";
      sensorData += key;
      sensorData += ",";
      sensorData += value;	

  } 
   
  void S4::addData(char* key, char* value)
  {     
	  sensorData += ",";
      sensorData += key;
      sensorData += ",";
      sensorData += value;	 

  }
  void S4::addData(char* key, long value)
  {     
	  sensorData += ",";
      sensorData += key;
      sensorData += ",";
      sensorData += value;  	 
  }
  void S4::addData(char* key, unsigned int value)
  {      
	  sensorData += ",";
      sensorData += key;
      sensorData += ",";
      sensorData += value; 	 
  }   
  void S4::addData(char* key, double value, int precision)
  {      
	  sensorData += ",";
      sensorData += key;
      sensorData += ",";
      char output[10];
      dtostrf(value,2,precision,output);
      sensorData += output;	
  }
  extern int __bss_end;
  extern void *__brkval;
	int S4::memoryFree()
	{
	  int freeValue;

	  if ((int)__brkval == 0)
		 freeValue = ((int)&freeValue) - ((int)&__bss_end);
	  else
		freeValue = ((int)&freeValue) - ((int)__brkval);

	  return freeValue;
	}

